@@ -1,9 +1,6 @@
-package # hide from PAUSE
+package # hide from PAUSE 
     DBICTest::Schema::BooksInLibrary;
 
-use warnings;
-use strict;
-
 use base qw/DBICTest::BaseResult/;
 
 __PACKAGE__->table('books');
@@ -30,10 +27,6 @@ __PACKAGE__->add_columns(
 );
 __PACKAGE__->set_primary_key('id');
 
-__PACKAGE__->add_unique_constraint (['title']);
-
 __PACKAGE__->resultset_attributes({where => { source => "Library" } });
 
-__PACKAGE__->belongs_to ( owner => 'DBICTest::Schema::Owners', 'owner' );
-
 1;
